#version 450 core
#extension GL_GOOGLE_include_directive : require

#include "per_frame.inc"

layout(location = 0) in vec3 inPos;
layout(location = 1) in vec3 inNormal;
layout(location = 2) in vec3 inTangent;
layout(location = 3) in vec2 inUV;

layout(set = 0, binding = 0) uniform Frame {
    PerFrameData frame;
};

layout(set = 1, binding = 0) uniform sampler2D samplerDiffuse;
layout(set = 1, binding = 1) uniform sampler2D samplerNormal;

layout(location = 0) out vec4 outFragColor;

void main() {
    vec3 vertexNormal = normalize(inNormal);
    vec3 vertexTangent = normalize(inTangent);
    vec3 vertexBitangent = cross(vertexTangent, vertexNormal);
    mat3 tangentToWorldSpace = mat3(vertexTangent, vertexBitangent, vertexNormal);
    vec3 normalMapNormal = texture(samplerNormal, inUV).xyz * 2.0 - 1.0;
    normalMapNormal.z = 1.0 - sqrt(normalMapNormal.x * normalMapNormal.x + normalMapNormal.y * normalMapNormal.y);
    vec3 normal = tangentToWorldSpace * normalMapNormal;
    vec4 diffuse = texture(samplerDiffuse, inUV);
    float nDotL = dot(normal, normalize(vec3(0.1, 0.14, 1.0)));

    outFragColor = vec4(max(nDotL, 0.23) * diffuse.rgb, diffuse.a);
}
