#pragma once

#include "stereo_strategy.hpp"

namespace glsl {
using namespace glm;
#include "res/dpr/dpr_parameters.inc"
}

class depth_peeling_reprojection_stereo final : public stereo_strategy {
public:
    depth_peeling_reprojection_stereo(vr_app* app);
    ~depth_peeling_reprojection_stereo();

    bool create() override;
    void render(VkCommandBuffer command_buffer, lava::index frame) override;
    void render_companion_window(VkCommandBuffer command_buffer) override;
    void update() override final;
    void draw_imgui() override;

private:
    lava::uv2 framebuffer_size_;
    lava::pipeline_layout::ptr pipeline_layout_;
    lava::render_pass::ptr render_passes_[2];
    VkDescriptorSet descriptor_sets_[2];
    lava::index current_render_pass_;
    lava::descriptor::ptr descriptor_;
    lava::descriptor::pool::ptr descriptor_pool_;

    lava::image_array::ptr color_layers_;
    lava::attachment::ptr color_layers_attachment_;

    lava::image_array::ptr depth_stencil_layers_[2];
    lava::attachment::ptr depth_stencil_layers_attachment_[2];

    lava::image::ptr second_layer_image_;
    lava::attachment::ptr second_layer_attachment_;

    lava::render_pass::ptr create_render_pass(lava::index render_pass_index);

    lava::mesh::ptr reprojection_mesh_;
    lava::graphics_pipeline::ptr reprojection_pipeline_;
    lava::pipeline_layout::ptr reprojection_pipeline_layout_;
    lava::descriptor::ptr reprojection_pipeline_descriptor_;
    VkDescriptorSet reprojection_pipeline_descriptor_set_[2];
    lava::render_pass::ptr reprojection_render_pass_;
    glsl::DprParameters params;
    lava::buffer::ptr reproject_ubo_;
    bool create_reprojection_render_pass();

    lava::mesh::ptr quad_mesh_;
    lava::graphics_pipeline::ptr companion_window_pipeline_;
    lava::pipeline_layout::ptr companion_window_pipeline_layout_;
    lava::descriptor::ptr companion_window_pipeline_descriptor_;
    VkDescriptorSet companion_window_pipeline_descriptor_set_;

    bool setup_companion_window_pipeline();

    uint32_t layers_to_draw_ = 0b01 | 0b10;
};
