#pragma once

#include <memory>
#include <liblava/lava.hpp>
#include <assimp/Importer.hpp>
#include <assimp/scene.h>

namespace glsl {
using namespace glm;
#include "res/dpr/mesh_data.inc"
}

struct scene_material {
    using ptr = std::shared_ptr<scene_material>;

    lava::texture::ptr diffuse;
    lava::texture::ptr normal;
    VkDescriptorSet descriptor_set;
};

struct scene_mesh {
    lava::mesh::ptr mesh;
    VkDescriptorSet descriptor_set;
    size_t node_index;
    size_t material_index;
};

struct scene_animation_channel {
    size_t node_index;
    std::vector<std::pair<float, glm::vec3>> position_frames;
    std::vector<std::pair<float, glm::quat>> rotation_frames;
    std::vector<std::pair<float, glm::vec3>> scaling_frames;
};

struct scene_animation {
    float duration;
    float ticks_per_second;
    std::vector<scene_animation_channel> channels;
    std::string name;
};

struct scene_camera {
    size_t node_index;
    glm::mat4 projection_matrix;
    glm::mat4 view_matrix;
    std::string name;
    float near_plane;
    float far_plane;
};

class scene {
public:
    using ptr = std::shared_ptr<scene>;

    struct node {
      glm::mat4 initial_local_transform;
      glm::mat4 current_local_transform;
      glm::mat4 current_global_transform;
      size_t parent_index;
    };

    scene() {}

    static void set_vertex_input(lava::graphics_pipeline* pipeline);

    void update(float delta_time);

    const scene_animation* get_active_animation() const { return active_animation_index_.has_value() ? &animations[*active_animation_index_] : nullptr; }
    scene_animation* get_active_animation() { return active_animation_index_.has_value() ? &animations[*active_animation_index_] : nullptr; }
    std::optional<std::size_t> get_active_animation_index() const { return active_animation_index_; }
    void set_animation(size_t animation_index);
    void clear_animation();

public:
    std::vector<node> nodes;
    std::vector<scene_mesh> meshes;
    std::vector<scene_material::ptr> materials;

    lava::buffer::ptr mesh_data_buffer;
    std::vector<glsl::MeshData> mesh_data;

    lava::descriptor::pool::ptr descriptor_pool_;
    lava::descriptor::ptr material_descriptor;
    lava::descriptor::ptr mesh_descriptor;
    lava::texture::ptr dummy_texture;

    float current_time = 0.0f;
    bool loop_ = true;
    std::vector<scene_animation> animations;
    std::optional<size_t> active_animation_index_;

    std::vector<scene_camera> cameras;

    void reset_transforms();
};

scene::ptr load_scene(lava::device_ptr device, lava::name filename);
