#pragma once

#include <liblava/lava.hpp>
#include <openvr.h>
#include <optional>
#include <memory>
#include "stereo_strategy.hpp"
#include "naive_stereo.hpp"
#include "multiview_stereo.hpp"
#include "depth_peeling_reprojection_stereo.hpp"
#include "scene.hpp"
#include "frame_capture.hpp"
#include "pass_timer.hpp"

class vr_app {
public:
    vr_app(lava::name name, argh::parser cmd_line = {});

    auto device() const { return app_->device; }
    auto& staging() { return app_->staging; }
    lava::camera& camera() { return app_->camera; }
    auto scene() { return scene_; }
    auto& frame_capture() { return frame_capture_; }
    auto& pass_timer() { return pass_timer_; }

    auto per_frame_descriptor() const { return per_frame_descriptor_; }

    bool setup();
    lava::frame::result run() { return app_->run(); }

    stereo_framebuffer::ptr get_framebuffer_for_eye(vr::EVREye eye) const {
        static_assert(vr::EVREye::Eye_Left == 0);
        static_assert(vr::EVREye::Eye_Right == 1);
        return framebuffers_[static_cast<int>(eye)];
    }
    stereo_framebuffer::ptr left_eye_framebuffer() const { return framebuffers_[0]; }
    stereo_framebuffer::ptr right_eye_framebuffer() const { return framebuffers_[1]; }

private:
    struct shutdown_openvr {
        void operator()(vr::IVRSystem*) { vr::VR_Shutdown(); }
    };
    std::unique_ptr<vr::IVRSystem, shutdown_openvr> vr_system = nullptr;
    std::optional<lava::app> app_;
    lava::descriptor::pool::ptr descriptor_pool_;

    lava::descriptor::ptr per_frame_descriptor_;
    stereo_framebuffer::ptr framebuffers_[2];

    lava::texture::ptr test_texture_;
    lava::mesh::ptr quad_mesh_;
    lava::graphics_pipeline::ptr companion_window_pipeline_;
    lava::pipeline_layout::ptr companion_window_pipeline_layout_;
    lava::descriptor::ptr companion_window_pipeline_descriptor_;
    VkDescriptorSet companion_window_pipeline_descriptor_set_;

public:
    VkSampler sampler;

private:
    std::array<std::unique_ptr<stereo_strategy>, 2> stereo_strategies_;
    int selected_strategy_ = 1;
    lava::graphics_pipeline::ptr current_companion_window_pipeline_;
    // std::unique_ptr<naive_stereo> naive_stereo_;
    // // std::unique_ptr<multiview_stereo> multiview_stereo_;
    // std::unique_ptr<depth_peeling_reprojection_stereo> depth_peeling_reprojection_stereo_;
    // stereo_strategy* active_strategy_ = nullptr;

    std::string scene_path_;
    ::scene::ptr scene_;
    ::frame_capture frame_capture_ = {"captures"};
    ::pass_timer pass_timer_ = {"statistics"};

    bool setup_companion_window_pipeline();
    stereo_framebuffer::ptr create_framebuffer(glm::uvec2 size, vr::EVREye eye);
    void calculate_projection_matrices();
    void calculate_projection_matrix(vr::EVREye eye);

    static constexpr int FRAME_TIMES_BUFFER_SIZE = 256;
    std::array<float, FRAME_TIMES_BUFFER_SIZE> frame_times_;
    int frame_time_smoothing_ = 4;
    std::size_t frame_times_begin_ = 0;
    float animation_playback_speed_ = 1.0f;
    std::optional<std::size_t> selected_camera_;
};
